package org.codecop.basic
/**
 * Variables are just available as setters in the <code>NewStatement</code> of
 * the Parser trait and use an expression as parameter. A piece of code is put 
 * into the program space. At runtime the setter calls the expression and puts
 * the value into the runtime's variables registry. The getters are only needed
 * for the compiler to accept the setters as overwritten accessors.
 * @author Peter Kofler
 * @param T is the return type of <code>addToMemory</code> for subclasses.
 */
trait VariableAssignment[T] {

   protected def memory:LinesOfCode
   protected def addToMemory(code: =>Unit):T
   
   protected var lastParsedAssignment = ""
  
   private def assign(name:String, d:DoubleExpression) = {
      lastParsedAssignment = name
      addToMemory {
         // println("exec. " + name + "=" + d.value)
         memory.set(name, d.value)
      }
   }
   private def assignString(name:String, s:StringExpression) = {
      lastParsedAssignment = name
      addToMemory {
         // println("exec. " + name + "=" + s.value)
         memory.setString(name, s.value)
      }
   }
   private def unused = throw new Error("never used")
   
   // ---------- variable assignment ----------

   def X_=(d:DoubleExpression) = assign("X", d)
   def X:DoubleExpression = unused

   def X$_=(s:StringExpression) = assignString("X$", s)
   def X$:StringExpression = unused

   // TODO for all simple variables A and A$
   def Y_=(d:DoubleExpression) = assign("Y", d)
   def Y:DoubleExpression = unused
   def Z_=(d:DoubleExpression) = assign("Z", d)
   def Z:DoubleExpression = unused

}
