package org.codecop.basic
/**
 * Variables are just available as getters in the Parser trait and produce
 * expressions to be consumed by methods of the <code>NewStatement</code>.
 * Expressions itself are evaluated lazy during runtime because the commands
 * using them are evaluated lazily.
 * @author Peter Kofler
 */
trait VariableEvaluation {

   /** Interaction with the program space. */
   protected def memory:LinesOfCode

   class VariableDoubleExpression(override val fromVariable:String) extends DoubleExpression {
      override def value = memory.get(fromVariable)
   }
   class VariableStringExpression(override val fromVariable:String) extends StringExpression {
      override def value = memory.getString(fromVariable)
   }

   // ---------- variable evaluation ----------

   def X = new VariableDoubleExpression("X")
   def X$ = new VariableStringExpression("X$")

   // TODO for all simple variables A and A$
   def Y = new VariableDoubleExpression("Y")
   def Z = new VariableDoubleExpression("Z")
   
}
